import processing.core.*; 
import processing.data.*; 
import processing.opengl.*; 

import toxi.physics2d.*; 
import toxi.physics2d.behaviors.*; 
import toxi.geom.*; 
import de.bezier.data.*; 
import processing.pdf.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class DataViz extends PApplet {

//Created by Ryan S. Ritter

//Import ToxicLibs



//Import XlsReader
 
//Import PDF


// Reference to physics world
VerletPhysics2D physics;
// Create new positive attraction behavior for center of display
AttractionBehavior center;
WordParticle anchor;
// Total number of particles
int NUM_PARTICLES;
// Declare variables for reading data files
XlsReader data;
int rowCount;
int row;
int i;
int countMin;
int countMax;
float freqMin;
float freqMax;
PFont font;
PFont titleFont;
String dictionary;
String caption;
int from; //color of lowest data point
int to; //color of highest data point
boolean record;


public void setup() {
  size(1000, 600);
  smooth();
  frameRate(30);
  i = 1;
  
  font = createFont("Droid Sans", 12, true);
  titleFont = createFont("Droid Sans", 24, true);

  //Set map interpolation colors
  from = color(204, 102, 0);
  to = color(0, 102, 203);

  //For now, change only these values to get results of different dictionaries
  data = new XlsReader(this, "insightwordcount.xls");
  dictionary = "Insight";
  caption = "Overall, Atheist followers use more Insight words than Christian followers.\nThis visualization shows divergence within the dictionary.";
  rowCount = data.getLastRowNum();
  NUM_PARTICLES = rowCount;
  countMinMax(data, rowCount);
  freqMinMax(data, rowCount);
  initPhysics();
}

public void addParticle(int _i, int count, float freq) {
  Vec2D f;
  String w;
  float d;
  float r;
  float percent; //percentage for lerpColor
  w = data.getString(_i, 0);
  float _force = map(freq, freqMin, freqMax, -2, 2);  //Map the force to a value between -1 and 1 to get particles trending toward left or right
  f = new Vec2D(_force, 0); //Set force along x-axis
  d = map(freq, freqMin, freqMax, -300, 300);  //Map the spring resting distance to a value between 0 and 400 to control distance from center
  d = abs(d);
  r = map(count, countMin, countMax, 30, 60);  //Map the radius of ellipse, based on overall word count, to a value between 25 and 80
  percent = norm(freq, freqMin, freqMax);
  //println("Word: " + w);
  //println("Force: " + f);
  //println("Distance: " + d);
  //println("Radius: " + r);
  Vec2D pos = new Vec2D(width/2, 150);
  WordParticle p = new WordParticle(pos, d, r, f, w, percent);
  physics.addParticle(p);
}


public void draw() {
//  if (record) {
//    // Note that #### will be replaced with the frame number. Fancy!
//    beginRecord(PDF, "frame-####.pdf");
//  }
  ellipseMode(RADIUS);
  background(255);
  noStroke();
  if (physics.particles.size() < NUM_PARTICLES+1) {
    addParticle(i, data.getInt(i, 1), data.getFloat(i, 2));
    //println(i);
    i++;
  }
  physics.update();
  for (VerletParticle2D p : physics.particles) {
    WordParticle wp = (WordParticle)p;
    fill(wp.particleColor); 
    ellipse(p.x, p.y, wp.size, wp.size);
    textFont(font);
    textAlign(CENTER);
    fill(255);
    text(wp.word, p.x, p.y); 
    physics.addBehavior(new AttractionBehavior(p, wp.size*2.2f, -.01f, 0.001f));
    VerletSpring2D spring = new VerletSpring2D(anchor, p, wp.distance, 0.001f);  
    physics.addSpring(spring);
    p.addForce(wp.force);
  }
  //Draw interpolation legend
  for (int j = 100; j < 901; j++) {
    float legend = norm(j, 100, 900);   
    stroke(lerpColor(from, to, legend, RGB));
    line(j, 550, j, 560);
  }
  fill(0);
  textAlign(LEFT);
  //text("Press:\n  I = Insight dictionary\nN = Negative Emotion dictionary\nP = Positive Emotion dictionary\nR = Religion dictionary\nS = Social Dictionary",5,100); 
  text("Words used relatively more by\nAtheist Followers", 100, 575);
  textAlign(RIGHT);
  text("Words used relatively more by\nChristian Followers", 900, 575);
  //text("Visualization based on ~2 Million Tweets from\n~24,000 Twitter users following either well-known\nAtheists (e.g., Richard Dawkins) or\nChristians (e.g., Rick Warren)",995,50);
  textAlign(CENTER);
  text("Press Key for Dictionary: I = Insight, N = Negative Emotion, P = Positive Emotion, R = Religion, S = Social Processes",width/2,125);
  text(caption, width/2, 525);
  textFont(titleFont);
  text("Following Christians vs. Atheists on Twitter:\nAn Investigation of Happiness", width/2, 50);
  text(dictionary, width/2, 500);

  if (record) {
    endRecord();
    record = false;
  }
}

class WordParticle extends VerletParticle2D {
  float size;
  float distance;
  Vec2D force;
  String word;
  int particleColor;

  WordParticle(Vec2D p, float d, float r, Vec2D f, String w, float percent) {
    super(p);
    this.size = r;
    this.distance = d;
    this.force = f;
    this.word = w;
    this.particleColor = lerpColor(from, to, percent, RGB);
  }
}


public void keyPressed() {
  if (key == 'r' || key == 'R') {
    physics.clear();
    initPhysics();
    dictionary = "Religion";
    caption = "Overall, Christian followers use more Religion words than Atheist followers.\nThis visualization shows divergence within the dictionary.";
    data = new XlsReader(this, "religwordcount.xls");
    rowCount = data.getLastRowNum();
    NUM_PARTICLES = rowCount;
    i = 1;
    countMinMax(data, rowCount);
    freqMinMax(data, rowCount);
  }
  else if (key == 'i' || key == 'I') {
    physics.clear();
    initPhysics();
    dictionary = "Insight";
    caption = "Overall, Atheist followers use more Insight words than Christian followers.\nThis visualization shows divergence within the dictionary.";
    data = new XlsReader(this, "insightwordcount.xls");
    rowCount = data.getLastRowNum();
    NUM_PARTICLES = rowCount;
    i = 1;
    countMinMax(data, rowCount);
    freqMinMax(data, rowCount);
  }
  else if (key == 'n' || key == 'N') {
    physics.clear();
    initPhysics();
    dictionary = "Negative Emotion";
    caption = "Overall, Atheist followers use more Negative Emotion words than Christian followers.\nThis visualization shows divergence within the dictionary.";
    data = new XlsReader(this, "negemowordcount.xls");
    rowCount = data.getLastRowNum();
    NUM_PARTICLES = rowCount;
    i = 1;
    countMinMax(data, rowCount);
    freqMinMax(data, rowCount);
  }
  else if (key == 'p' || key == 'P') {
    physics.clear();
    initPhysics();
    dictionary = "Positive Emotion";
    caption = "Overall, Christian followers use more Positive Emotion words than Atheist followers.\nThis visualization shows divergence within the dictionary.";
    data = new XlsReader(this, "posemowordcount.xls");
    rowCount = data.getLastRowNum();
    NUM_PARTICLES = rowCount;
    i = 1;
    countMinMax(data, rowCount);
    freqMinMax(data, rowCount);
  }
  else if (key == 's' || key == 'S') {
    physics.clear();
    initPhysics();
    dictionary = "Social Processes";
    caption = "Overall, Christian followers use more Social words than Atheist followers.\nThis visualization shows divergence within the dictionary.";
    data = new XlsReader(this, "socialwordcount.xls");
    rowCount = data.getLastRowNum();
    NUM_PARTICLES = rowCount;
    i = 1;
    countMinMax(data, rowCount);
    freqMinMax(data, rowCount);
  }
}

public void countMinMax(XlsReader _data, int _r) {
  XlsReader data = _data;
  int rows = _r;
  //Find Min & Max word count values
  countMin = 1000000;  //set countMin to arbitrarily high value so that it is not zero by default
  countMax = 0;
  for (int row = 1; row < rows+1; row++) {
    int value = data.getInt(row, 1);
    if (value > countMax) {
      countMax = value;
    }
    if (value < countMin) {
      countMin = value;
    }
  }
}

public void freqMinMax(XlsReader _data, int _r) {
  XlsReader data = _data;
  int rows = _r;
  //Find Min & Max word frequency values
  freqMin = 1000000;
  freqMax = 0;
  for (int row = 1; row < rows+1; row++) {
    float value = data.getFloat(row, 2);
    if (value > freqMax) {
      freqMax = value;
    }
    if (value < freqMin) {
      freqMin = value;
    }
  }
}

public void initPhysics() {  
  // Initialize the physics
  physics=new VerletPhysics2D();
  physics.setDrag(0.3f);
  // Set the physics world to constitute middle third of the screen
  physics.setWorldBounds(new Rect(0, 0, width, height));
  // Create positive attraction behavior toward center of the graph
  //center = new AttractionBehavior(new Vec2D(width/2, height/2), width, 0.9f);
  //physics.addBehavior(center);
  // Create an anchor particle in the center of the graph.  Springs will be attached from this anchor to all word particles.
  anchor = new WordParticle(new Vec2D(width/2, height/2), 0, 0, new Vec2D(0, 0), "", 0);
  anchor.lock();
  physics.addParticle(anchor);
}

//void mousePressed() {
//  record = true;
//}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "DataViz" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
